@@ -1,6 +1,7 @@
 import datetime
 
-from django.db import models
+from django.db import models, connections
+from django.db.models.query import QuerySet
 
 from rapidsms.models import Contact, Connection
 
@@ -13,6 +14,7 @@ STATUS_CHOICES = (
     ("H", "Handled"),
 
     ("P", "Processing"),
+    ("L", "Locked"),
 
     ("Q", "Queued"),
     ("S", "Sent"),
@@ -22,6 +24,25 @@ STATUS_CHOICES = (
     ("E", "Errored")
 )
 
+#
+# Allows us to use SQL to lock a row when setting it to 'locked'.  Without this
+# in a multi-process environment like Gunicorn we'll double send messages.
+#
+# See: https://coderanger.net/2011/01/select-for-update/
+#
+
+class ForUpdateQuerySet(QuerySet):
+    def for_single_update(self):
+        if 'sqlite' in connections[self.db].settings_dict['ENGINE'].lower():
+            # Noop on SQLite since it doesn't support FOR UPDATE
+            return self
+        sql, params = self.query.get_compiler(self.db).as_sql()
+        return self.model._default_manager.raw(sql.rstrip() + ' LIMIT 1 FOR UPDATE', params)
+
+class ForUpdateManager(models.Manager):
+    def get_query_set(self):
+        return ForUpdateQuerySet(self.model, using=self._db)
+
 class Message(models.Model):
     connection = models.ForeignKey(Connection, related_name='messages')
     text       = models.TextField()
@@ -29,7 +50,10 @@ class Message(models.Model):
     status     = models.CharField(max_length=1, choices=STATUS_CHOICES)
     date       = models.DateTimeField(auto_now_add=True)
 
-    in_response_to = models.ForeignKey('self', related_name='responses', null=True)
+    in_response_to = models.ForeignKey('self', related_name='responses', null=True, blank=True)
+
+    # set our manager to our update manager
+    objects = ForUpdateManager()
 
     def __unicode__(self):
         # crop the text (to avoid exploding the admin)
@@ -44,3 +68,5 @@ class Message(models.Model):
                     contact=self.connection.identity, backend=self.connection.backend.name,
                     direction=self.direction, status=self.status, text=self.text,
                     date=self.date.isoformat())
+
+
